/*!
    \file    gd32e23x_it.c
    \brief   interrupt service routines

    \version 2024-02-22, V2.1.0, firmware for GD32E23x
*/

/*
    Copyright (c) 2024, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include "gd32e23x_it.h"
#include "main.h"
#include "systick.h"

/*!
    \brief      this function handles NMI exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void NMI_Handler(void)
{
    /* if NMI exception occurs, go to infinite loop */
    while(1) {
    }
}

/*!
    \brief      this function handles HardFault exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void HardFault_Handler(void)
{
    /* if Hard Fault exception occurs, go to infinite loop */
    while(1) {
    }
}

/*!
    \brief      this function handles SVC exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void SVC_Handler(void)
{
    /* if SVC exception occurs, go to infinite loop */
    while(1) {
    }
}

/*!
    \brief      this function handles PendSV exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void PendSV_Handler(void)
{
    /* if PendSV exception occurs, go to infinite loop */
    while(1) {
    }
}

/*!
    \brief      this function handles SysTick exception
    \param[in]  none
    \param[out] none
    \retval     none
*/
void SysTick_Handler(void)
{
}

void TIMER16_IRQHandler(void) {
    if (timer_interrupt_flag_get(TIMER16, TIMER_INT_FLAG_UP) == SET)
    {
        timer_interrupt_flag_clear(TIMER16, TIMER_INT_FLAG_UP);
        static uint8_t led_status = 0;
        if (led_status)
        {
            //! turn on led & reconfig timer13 period to 19000(1900ms)
            gpio_bit_write(GPIOA, GPIO_PIN_9, RESET);
            timer_autoreload_value_config(TIMER16, 19200);
        } else {
            //! turn off led & reconfig timer13 period to 1000(100ms)
            gpio_bit_write(GPIOA, GPIO_PIN_9, SET);
            timer_autoreload_value_config(TIMER16, 800);
        }
        led_status = !led_status;
    }
}

void TIMER14_IRQHandler(void)
{
    if (timer_flag_get(TIMER14, TIMER_FLAG_CH0))
    {
        timer_flag_clear(TIMER14, TIMER_FLAG_CH0);

        uint32_t capture_value = timer_channel_capture_value_register_read(TIMER14, TIMER_CH_0);

        timer_disable(TIMER14);
    }
}

void TIMER15_IRQHandler(void) {
    if (timer_interrupt_flag_get(TIMER15, TIMER_INT_FLAG_UP) == SET)
    {
        timer_interrupt_flag_clear(TIMER15, TIMER_INT_FLAG_UP);

        exti_interrupt_enable(EXTI_0);
        timer_enable(TIMER14);

        /*
         * GPIO Toggle will be delete when the project is ready
         */
        gpio_bit_toggle(GPIOA, GPIO_PIN_7);
        /*******************************************************/

        timer_disable(TIMER15);
    }
}

void EXTI0_1_IRQHandler(void) {
    if (exti_interrupt_flag_get(EXTI_0) == SET)
    {
        exti_interrupt_flag_clear(EXTI_0);

        /*
         * GPIO Toggle will be delete when the project is ready
         */
        gpio_bit_toggle(GPIOA, GPIO_PIN_5);
        /*******************************************************/

        // timer_event_software_generate(TIMER14, TIMER_EVENT_SRC_CH0G);

        exti_interrupt_disable(EXTI_0);
    }
}